/*
 * NT Service Status Control Tool
 * 
 * Copyright (c) RegExLab.com
 * 
 * 2006.03.27 Chongqing China.
 */
package com.regexlab.j2e;

/**
 * Send command to service manager
 * 
 * @author sswater shi
 */
public class ServiceStatusManager
{
    /** Service status constants */
    public final static int STATUS_PAUSED  = 0x00000007;
    /** Service status constants */
    public final static int STATUS_RUNNING = 0x00000004;

    /** Whether runs as service, will be set by frame, do not modify it by yourself */
    public static boolean isRunAsService = false; 

    /**
     * Set service status, this will not leads to onPause/onContinue event.
     * @param status New status to set, use the constants before
     */
    public static void setServiceStatus(int status)
    {
        if( isRunAsService ) nativeSetServiceStatus(status);
    }

    /**
     * Set status callback handler
     * @param handler Callback instance to do events
     */
    public static void setServiceStatusHandler(ServiceStatusHandler handler)
    {
        if( isRunAsService ) nativeSetServiceStatusHandler(handler);
    }

    /*
     * Native interface, communicate with the frame
     */
    private static native void nativeSetServiceStatus(int status);
    private static native void nativeSetServiceStatusHandler(ServiceStatusHandler handler);

    static
    {
        String j2epath = System.getProperty("j2e.app.path");
        if( j2epath != null )
            System.load( j2epath );
    }
}
